package sun.invoke.util;

import java.lang.invoke.MethodType;
import sun.invoke.empty.Empty;

public class VerifyType
{
    private VerifyType() {
    }
    
    public static boolean isNullConversion(Class<?> clazz, Class<?> clazz2, boolean b) {
        if (clazz == clazz2) {
            return true;
        }
        if (!b) {
            if (clazz2.isInterface()) {
                clazz2 = Object.class;
            }
            if (clazz.isInterface()) {
                clazz = Object.class;
            }
            if (clazz == clazz2) {
                return true;
            }
        }
        if (isNullType(clazz)) {
            return !clazz2.isPrimitive();
        }
        if (!clazz.isPrimitive()) {
            return clazz2.isAssignableFrom(clazz);
        }
        if (!clazz2.isPrimitive()) {
            return false;
        }
        Wrapper forPrimitiveType = Wrapper.forPrimitiveType(clazz);
        if (clazz2 == Integer.TYPE) {
            return forPrimitiveType.isSubwordOrInt();
        }
        Wrapper forPrimitiveType2 = Wrapper.forPrimitiveType(clazz2);
        return forPrimitiveType.isSubwordOrInt() && forPrimitiveType2.isSubwordOrInt() && (forPrimitiveType2.isSigned() || !forPrimitiveType.isSigned()) && forPrimitiveType2.bitWidth() > forPrimitiveType.bitWidth();
    }
    
    public static boolean isNullReferenceConversion(Class<?> clazz, Class<?> clazz2) {
        assert !clazz2.isPrimitive();
        return clazz2.isInterface() || isNullType(clazz) || clazz2.isAssignableFrom(clazz);
    }
    
    public static boolean isNullType(Class<?> clazz) {
        return clazz == Void.class || clazz == Empty.class;
    }
    
    public static boolean isNullConversion(MethodType methodType, MethodType methodType2, boolean b) {
        if (methodType == methodType2) {
            return true;
        }
        int parameterCount = methodType.parameterCount();
        if (parameterCount != methodType2.parameterCount()) {
            return false;
        }
        int i = 0;
        while (i < parameterCount) {
            if (!isNullConversion(methodType.parameterType(i), methodType2.parameterType(i), b)) {
                return false;
            }
            ++i;
        }
        return isNullConversion(methodType2.returnType(), methodType.returnType(), b);
    }
    
    public static int canPassUnchecked(Class<?> clazz, Class<?> clazz2) {
        if (clazz == clazz2) {
            return 1;
        }
        if (clazz2.isPrimitive()) {
            if (clazz2 == Void.TYPE) {
                return 1;
            }
            if (clazz == Void.TYPE) {
                return 0;
            }
            if (!clazz.isPrimitive()) {
                return 0;
            }
            Wrapper forPrimitiveType = Wrapper.forPrimitiveType(clazz);
            Wrapper forPrimitiveType2 = Wrapper.forPrimitiveType(clazz2);
            if (forPrimitiveType.isSubwordOrInt() && forPrimitiveType2.isSubwordOrInt()) {
                if (forPrimitiveType.bitWidth() >= forPrimitiveType2.bitWidth()) {
                    return -1;
                }
                if (!forPrimitiveType2.isSigned() && forPrimitiveType.isSigned()) {
                    return -1;
                }
                return 1;
            }
            else if (clazz == Float.TYPE || clazz2 == Float.TYPE) {
                if (clazz == Double.TYPE || clazz2 == Double.TYPE) {
                    return -1;
                }
                return 0;
            }
            else {
                return 0;
            }
        }
        else {
            if (clazz.isPrimitive()) {
                return 0;
            }
            if (isNullReferenceConversion(clazz, clazz2)) {
                return 1;
            }
            return -1;
        }
    }
    
    public static boolean isSpreadArgType(Class<?> clazz) {
        return clazz.isArray();
    }
    
    public static Class<?> spreadArgElementType(Class<?> clazz, int n) {
        return clazz.getComponentType();
    }
}
